from pyiron_lammps.decorator import calculation
from pyiron_lammps.elastic import ElasticMatrixCalculator


def _run_simulation(structure, potential_dataframe, input_template, lmp):
    # write structure to LAMMPS
    lmp.interactive_structure_setter(
        structure=structure,
        units="metal",
        dimension=3,
        boundary=" ".join(["p" if coord else "f" for coord in structure.pbc]),
        atom_style="atomic",
        el_eam_lst=potential_dataframe.Species,
        calc_md=False,
    )

    # execute calculation
    for c in potential_dataframe.Config:
        lmp.interactive_lib_command(c)

    for l in input_template.split("\n"):
        lmp.interactive_lib_command(l)

    return lmp


@calculation
def optimize_structure(lmp, structure, potential_dataframe):
    lammps_input_template_minimize_cell = """\
fix ensemble all box/relax iso 0.0
variable thermotime equal 100
thermo_style custom step temp pe etotal pxx pxy pxz pyy pyz pzz vol
thermo_modify format float %20.15g
thermo ${thermotime}
min_style cg
minimize 0.0 0.0001 100000 10000000"""

    lmp = _run_simulation(
        structure=structure,
        potential_dataframe=potential_dataframe,
        input_template=lammps_input_template_minimize_cell,
        lmp=lmp,
    )

    # get final structure
    structure_copy = structure.copy()
    structure_copy.set_cell(lmp.interactive_cells_getter(), scale_atoms=True)
    structure_copy.positions = lmp.interactive_positions_getter()

    # clean memory
    lmp.interactive_lib_command("clear")
    return structure_copy


@calculation
def calculate_elastic_constants(
    lmp,
    structure,
    potential_dataframe,
    num_of_point=5,
    eps_range=0.005,
    sqrt_eta=True,
    fit_order=2,
):
    lammps_input_template_minimize_pos = """\
variable thermotime equal 100
thermo_style custom step temp pe etotal pxx pxy pxz pyy pyz pzz vol
thermo_modify format float %20.15g
thermo ${thermotime}
min_style cg
minimize 0.0 0.0001 100000 10000000"""

    # Generate structures
    calculator = ElasticMatrixCalculator(
        basis_ref=structure.copy(),
        num_of_point=num_of_point,
        eps_range=eps_range,
        sqrt_eta=sqrt_eta,
        fit_order=fit_order,
    )
    structure_dict = calculator.generate_structures()

    # run calculation
    energy_tot_lst = {}
    for key, struct in structure_dict.items():
        lmp = _run_simulation(
            lmp=lmp,
            structure=struct,
            potential_dataframe=potential_dataframe,
            input_template=lammps_input_template_minimize_pos,
        )
        energy_tot_lst[key] = lmp.interactive_energy_tot_getter()
        lmp.interactive_lib_command("clear")

    # fit
    calculator.analyse_structures(energy_tot_lst)
    return calculator._data["C"]


@calculation
def calculate_elastic_constants_with_minimization(
    lmp,
    structure,
    potential_dataframe,
    num_of_point=5,
    eps_range=0.005,
    sqrt_eta=True,
    fit_order=2,
):
    structure_optimized = optimize_structure(
        lmp=lmp, structure=structure, potential_dataframe=potential_dataframe
    )
    return calculate_elastic_constants(
        lmp=lmp,
        structure=structure_optimized,
        potential_dataframe=potential_dataframe,
        num_of_point=num_of_point,
        eps_range=eps_range,
        sqrt_eta=sqrt_eta,
        fit_order=fit_order,
    )



import random
import warnings
import itertools
from multiprocessing import cpu_count
from ase.data import atomic_numbers
from ase.atoms import Atoms
import numpy as np
from sqsgenerator import sqs_optimize
from typing import Dict, Optional, Union, Iterable


def chemical_formula(atoms: Atoms) -> str:
    def group_symbols():
        for species, same in itertools.groupby(atoms.get_chemical_symbols()):
            num_same = len(list(same))
            yield species if num_same == 1 else f"{species}{num_same}"

    return "".join(group_symbols())


def map_dict(f, d: Dict) -> Dict:
    return {k: f(v) for k, v in d.items()}


def mole_fractions_to_composition(
    mole_fractions: Dict[str, float], num_atoms: int
) -> Dict[str, int]:
    # if the sum of x is less than 1 - 1/n then we are missing at least one atoms
    if not (1.0 - 1 / num_atoms) < sum(mole_fractions.values()) < (1.0 + 1 / num_atoms):
        raise ValueError(
            "mole-fractions must sum up to one: {}".format(sum(mole_fractions.values()))
        )

    composition = map_dict(lambda x: x * num_atoms, mole_fractions)
    # check to avoid partial occupation -> x_i * num_atoms is not an integer number
    if any(
        map(
            lambda occupation: not float.is_integer(round(occupation, 1)),
            composition.values(),
        )
    ):
        # at least one of the specified species exhibits fractional occupation, we try to fix it by rounding
        composition_ = map_dict(lambda occ: int(round(occ)), composition)
        warnings.warn(
            "The current mole-fraction specification cannot be applied to {} atoms, "
            "as it would lead to fractional occupation. Hence, I have changed it from "
            '"{}" -> "{}"'.format(
                num_atoms,
                mole_fractions,
                map_dict(lambda n: n / num_atoms, composition_),
            )
        )
        composition = composition_

    # due to rounding errors there might be a difference of one atom
    actual_atoms = sum(composition.values())
    diff = actual_atoms - num_atoms
    if abs(diff) == 1:
        # it is not possible to distribute atoms equally e.g x_a = x_b = x_c = 1/3 on 32 atoms
        # we remove one randomly bet we inform the user
        removed_species = random.choice(tuple(composition))
        composition[removed_species] -= 1
        warnings.warn(
            'It is not possible to distribute the species properly. Therefore one "{}" atom was removed. '
            "This changes the input mole-fraction specification. "
            '"{}" -> "{}"'.format(
                removed_species,
                mole_fractions,
                map_dict(lambda n: n / num_atoms, composition),
            )
        )
    elif abs(diff) > 1:
        # something else is wrong with the mole-fractions input
        raise ValueError(
            "Cannot interpret mole-fraction dict {}".format(mole_fractions)
        )

    return composition


def remap_sro(species: Iterable[str], array: np.ndarray):
    # remaps computed short-range order parameters to style of sqsgenerator=v0.0.5
    species = tuple(sorted(species, key=lambda abbr: atomic_numbers[abbr]))
    return {
        "{}-{}".format(si, sj): array[:, i, j].tolist()
        for (i, si), (j, sj) in itertools.product(
            enumerate(species), enumerate(species)
        )
        if j >= i
    }


def remap_sqs_results(result):
    # makes new interface compatible with old one
    return result["structure"], remap_sro(
        set(result["structure"].get_chemical_symbols()), result["parameters"]
    )


def transpose(it):
    return zip(*it)


def get_sqs_structures(
    structure: Atoms,
    mole_fractions: Dict[str, Union[float, int]],
    weights: Optional[Dict[int, float]] = None,
    objective: Union[float, np.ndarray] = 0.0,
    iterations: Union[float, int] = 1e6,
    output_structures: int = 10,
    mode: str = "random",
    num_threads: Optional[int] = None,
    prefactors: Optional[Union[float, np.ndarray]] = None,
    pair_weights: Optional[np.ndarray] = None,
    rtol: Optional[float] = None,
    atol: Optional[float] = None,
    which: Optional[Iterable[int]] = None,
    shell_distances: Optional[Iterable[int]] = None,
    minimal: Optional[bool] = True,
    similar: Optional[bool] = True,
):
    composition = mole_fractions_to_composition(mole_fractions, len(structure))

    settings = dict(
        atol=atol,
        rtol=rtol,
        mode=mode,
        which=which,
        structure=structure,
        prefactors=prefactors,
        shell_weights=weights,
        iterations=iterations,
        composition=composition,
        pair_weights=pair_weights,
        target_objective=objective,
        shell_distances=shell_distances,
        threads_per_rank=num_threads or cpu_count(),
        max_output_configurations=output_structures,
    )
    # not specifying a parameter in settings causes sqsgenerator to choose a "sensible" default,
    # hence we remove all entries with a None value

    results, timings = sqs_optimize(
        {param: value for param, value in settings.items() if value is not None},
        minimal=minimal,
        similar=similar,
        make_structures=True,
        structure_format="ase",
    )

    structures, sro_breakdown = transpose(map(remap_sqs_results, results.values()))
    num_iterations = iterations
    cycle_time = np.average(list(map_dict(np.average, timings).values()))
    return structures, sro_breakdown, num_iterations, cycle_time



import pandas
from pathlib import Path
import os
from ase.atoms import Atoms


class PotentialAbstract(object):
    """
    The PotentialAbstract class loads a list of available potentials and sorts them. Afterwards the potentials can be
    accessed through:
        PotentialAbstract.<Element>.<Element> or PotentialAbstract.find_potentials_set({<Element>, <Element>}

    Args:
        potential_df:
        default_df:
        selected_atoms:
    """

    def __init__(self, potential_df, default_df=None, selected_atoms=None):
        self._potential_df = potential_df
        self._default_df = default_df
        if selected_atoms is not None:
            self._selected_atoms = selected_atoms
        else:
            self._selected_atoms = []

    def find(self, element):
        """
        Find the potentials

        Args:
            element (set, str): element or set of elements for which you want the possible LAMMPS potentials

        Returns:
            list: of possible potentials for the element or the combination of elements

        """
        if isinstance(element, set):
            element = element
        elif isinstance(element, list):
            element = set(element)
        elif isinstance(element, str):
            element = set([element])
        else:
            raise TypeError("Only, str, list and set supported!")
        return self._potential_df[
            [
                True if set(element).issubset(species) else False
                for species in self._potential_df["Species"].values
            ]
        ]

    def find_by_name(self, potential_name):
        mask = self._potential_df["Name"] == potential_name
        if not mask.any():
            raise ValueError(
                "Potential '{}' not found in database.".format(potential_name)
            )
        return self._potential_df[mask]

    def list(self):
        """
        List the available potentials

        Returns:
            list: of possible potentials for the element or the combination of elements
        """
        return self._potential_df

    def __getattr__(self, item):
        return self[item]

    def __getitem__(self, item):
        potential_df = self.find(element=item)
        selected_atoms = self._selected_atoms + [item]
        return PotentialAbstract(
            potential_df=potential_df,
            default_df=self._default_df,
            selected_atoms=selected_atoms,
        )

    def __str__(self):
        return str(self.list())

    @staticmethod
    def _get_potential_df(file_name_lst, resource_path):
        """

        Args:
            plugin_name (str):
            file_name_lst (set):
            resource_path (str):

        Returns:
            pandas.DataFrame:
        """
        for path, folder_lst, file_lst in os.walk(resource_path):
            for periodic_table_file_name in file_name_lst:
                if (
                    periodic_table_file_name in file_lst
                    and periodic_table_file_name.endswith(".csv")
                ):
                    return pandas.read_csv(
                        os.path.join(path, periodic_table_file_name),
                        index_col=0,
                        converters={
                            "Species": lambda x: x.replace("'", "")
                            .strip("[]")
                            .split(", "),
                            "Config": lambda x: x.replace("'", "")
                            .replace("\\n", "\n")
                            .strip("[]")
                            .split(", "),
                            "Filename": lambda x: x.replace("'", "")
                            .strip("[]")
                            .split(", "),
                        },
                    )
        raise ValueError("Was not able to locate the potential files.")


class LammpsPotentialFile(PotentialAbstract):
    """
    The Potential class is derived from the PotentialAbstract class, but instead of loading the potentials from a list,
    the potentials are loaded from a file.

    Args:
        potential_df:
        default_df:
        selected_atoms:
    """

    def __init__(
        self,
        potential_df=None,
        default_df=None,
        selected_atoms=None,
        resource_path=None,
    ):
        if potential_df is None:
            potential_df = self._get_potential_df(
                file_name_lst={"potentials_lammps.csv"},
                resource_path=resource_path,
            )
        super(LammpsPotentialFile, self).__init__(
            potential_df=potential_df,
            default_df=default_df,
            selected_atoms=selected_atoms,
        )
        self._resource_path = resource_path

    def default(self):
        if self._default_df is not None:
            atoms_str = "_".join(sorted(self._selected_atoms))
            return self._default_df[
                (self._default_df["Name"] == self._default_df.loc[atoms_str].values[0])
            ]
        return None

    def find_default(self, element):
        """
        Find the potentials

        Args:
            element (set, str): element or set of elements for which you want the possible LAMMPS potentials
            path (bool): choose whether to return the full path to the potential or just the potential name

        Returns:
            list: of possible potentials for the element or the combination of elements

        """
        if isinstance(element, set):
            element = element
        elif isinstance(element, list):
            element = set(element)
        elif isinstance(element, str):
            element = set([element])
        else:
            raise TypeError("Only, str, list and set supported!")
        element_lst = list(element)
        if self._default_df is not None:
            merged_lst = list(set(self._selected_atoms + element_lst))
            atoms_str = "_".join(sorted(merged_lst))
            return self._default_df[
                (self._default_df["Name"] == self._default_df.loc[atoms_str].values[0])
            ]
        return None

    def __getitem__(self, item):
        potential_df = self.find(element=item)
        selected_atoms = self._selected_atoms + [item]
        return LammpsPotentialFile(
            potential_df=potential_df,
            default_df=self._default_df,
            selected_atoms=selected_atoms,
            resource_path=self._resource_path,
        )


class PotentialAvailable(object):
    def __init__(self, list_of_potentials):
        self._list_of_potentials = {
            "pot_" + v.replace("-", "_").replace(".", "_"): v
            for v in list_of_potentials
        }

    def __getattr__(self, name):
        if name in self._list_of_potentials.keys():
            return self._list_of_potentials[name]
        else:
            raise AttributeError

    def __dir__(self):
        return list(self._list_of_potentials.keys())

    def __repr__(self):
        return str(dir(self))


def find_potential_file_base(path, resource_path_lst, rel_path):
    if path is not None:
        for resource_path in resource_path_lst:
            path_direct = os.path.join(resource_path, path)
            path_indirect = os.path.join(resource_path, rel_path, path)
            if os.path.exists(path_direct):
                return path_direct
            elif os.path.exists(path_indirect):
                return path_indirect
    raise ValueError(
        "Either the filename or the functional has to be defined.",
        path,
        resource_path_lst,
    )


def view_potentials(structure: Atoms, resource_path: str) -> pandas.DataFrame:
    """
    List all interatomic potentials for the given atomistic structure including all potential parameters.

    To quickly get only the names of the potentials you can use `list_potentials()` instead.

    Args:
        structure (Atoms): The structure for which to get potentials.
        resource_path (str): Path to the "lammps/potentials_lammps.csv" file

    Returns:
        pandas.Dataframe: Dataframe including all potential parameters.
    """
    list_of_elements = set(structure.get_chemical_symbols())
    return LammpsPotentialFile(resource_path=resource_path).find(list_of_elements)


def convert_path_to_abs_posix(path: str) -> str:
    """
    Convert path to an absolute POSIX path

    Args:
        path (str): input path.

    Returns:
        str: absolute path in POSIX format
    """
    return (
        Path(path.strip())
        .expanduser()
        .resolve()
        .absolute()
        .as_posix()
        .replace("\\", "/")
    )



import os
from ase.build import bulk
from pyiron_lammps.potential import view_potentials
from pyiron_lammps.sqs import get_sqs_structures
from pyiron_lammps.wrapper import PyironLammpsLibrary


def update_potential_paths(df_pot, resource_path):
    config_lst = []
    for row in df_pot.itertuples():
        potential_file_lst = row.Filename
        potential_file_path_lst = [
            os.path.join(resource_path, f) for f in potential_file_lst
        ]
        potential_dict = {os.path.basename(f): f for f in potential_file_path_lst}
        potential_commands = []
        for l in row.Config:
            l = l.replace("\n", "")
            for key, value in potential_dict.items():
                l = l.replace(key, value)
            potential_commands.append(l)
        config_lst.append(potential_commands)
    df_pot["Config"] = config_lst
    return df_pot


def generate_sqs_structure(structure_template, element_lst, count_lst):
    structures, sro_breakdown, num_iterations, cycle_time = get_sqs_structures(
        structure=structure_template,
        mole_fractions={
            el: c / len(structure_template) for el, c in zip(element_lst, count_lst)
        },
    )
    return structures


def get_lammps_engine(
    working_directory=None,
    cores=1,
    comm=None,
    logger=None,
    log_file=None,
    library=None,
    diable_log_file=True,
):
    return PyironLammpsLibrary(
        working_directory=working_directory,
        cores=cores,
        comm=comm,
        logger=logger,
        log_file=log_file,
        library=library,
        diable_log_file=diable_log_file,
    )


def get_ase_bulk(*args, **kwargs):
    return bulk(*args, **kwargs)


def get_potential_dataframe(structure, resource_path):
    return update_potential_paths(
        df_pot=view_potentials(structure=structure, resource_path=resource_path),
        resource_path=resource_path,
    )



import os
import unittest
import pyiron_lammps as pyr


def validate_elastic_constants(elastic_matrix):
    return [
        elastic_matrix[0, 0] > 200,
        elastic_matrix[1, 1] > 200,
        elastic_matrix[2, 2] > 200,
        elastic_matrix[0, 1] > 135,
        elastic_matrix[0, 2] > 135,
        elastic_matrix[1, 2] > 135,
        elastic_matrix[3, 3] > 100,
        elastic_matrix[4, 4] > 100,
        elastic_matrix[5, 5] > 100,
    ]


class TestParallelTwoCores(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        count_lst = [22, 22, 22, 21, 21]
        element_lst = ["Fe", "Ni", "Cr", "Co", "Cu"]
        potential = '2021--Deluigi-O-R--Fe-Ni-Cr-Co-Cu--LAMMPS--ipr1'
        resource_path = os.path.join(os.path.dirname(__file__), "static")

        # Generate SQS Structure
        structure = pyr.generate_sqs_structure(
            structure_template=pyr.get_ase_bulk("Al", cubic=True).repeat([3, 3, 3]),
            element_lst=element_lst,
            count_lst=count_lst
        )[0]

        # Select potential
        df_pot = pyr.get_potential_dataframe(
            structure=structure,
            resource_path=resource_path
        )

        # Assign variable
        cls.df_pot_selected = df_pot[df_pot.Name == potential].iloc[0]
        cls.structure = structure
        cls.resource_path = resource_path
        cls.potential = potential
        cls.count_lst = count_lst

    def test_structure(self):
        self.assertEqual(len(self.structure), sum(self.count_lst))

    def test_example_elastic_constants_parallel_cores_two(self):
        structure_opt_lst = pyr.optimize_structure_parallel(
            structure_list=[self.structure.copy()],
            potential_dataframe=self.df_pot_selected,
            cores=2
        )

        # Calculate Elastic Constants
        elastic_matrix = pyr.calculate_elastic_constants_parallel(
            structure_list=structure_opt_lst,
            potential_dataframe=self.df_pot_selected,
            num_of_point=5,
            eps_range=0.005,
            sqrt_eta=True,
            fit_order=2,
            cores=2
        )[0]

        self.assertEqual(len(structure_opt_lst[0]), sum(self.count_lst))
        self.assertTrue(all(validate_elastic_constants(elastic_matrix=elastic_matrix)))

    def test_example_elastic_constants_with_minimization_parallel_cores_two(self):
        elastic_matrix = pyr.calculate_elastic_constants_with_minimization_parallel(
            structure_list=[self.structure.copy()],
            potential_dataframe=self.df_pot_selected,
            num_of_point=5,
            eps_range=0.005,
            sqrt_eta=True,
            fit_order=2,
            cores=2
        )[0]
        self.assertTrue(all(validate_elastic_constants(elastic_matrix=elastic_matrix)))


if __name__ == '__main__':
    unittest.main()


import os
import unittest
import pyiron_lammps as pyr


def validate_elastic_constants(elastic_matrix):
    return [
        elastic_matrix[0, 0] > 200,
        elastic_matrix[1, 1] > 200,
        elastic_matrix[2, 2] > 200,
        elastic_matrix[0, 1] > 135,
        elastic_matrix[0, 2] > 135,
        elastic_matrix[1, 2] > 135,
        elastic_matrix[3, 3] > 100,
        elastic_matrix[4, 4] > 100,
        elastic_matrix[5, 5] > 100,
    ]


class TestIntegration(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        count_lst = [22, 22, 22, 21, 21]
        element_lst = ["Fe", "Ni", "Cr", "Co", "Cu"]
        potential = '2021--Deluigi-O-R--Fe-Ni-Cr-Co-Cu--LAMMPS--ipr1'
        resource_path = os.path.join(os.path.dirname(__file__), "static")

        # Generate SQS Structure
        structure = pyr.generate_sqs_structure(
            structure_template=pyr.get_ase_bulk("Al", cubic=True).repeat([3, 3, 3]),
            element_lst=element_lst,
            count_lst=count_lst
        )[0]

        # Select potential
        df_pot = pyr.get_potential_dataframe(
            structure=structure,
            resource_path=resource_path
        )

        # Assign variable
        cls.df_pot_selected = df_pot[df_pot.Name == potential].iloc[0]
        cls.structure = structure
        cls.resource_path = resource_path
        cls.potential = potential
        cls.count_lst = count_lst

    def test_structure(self):
        self.assertEqual(len(self.structure), sum(self.count_lst))

    def test_example_elastic_constants_with_one_executable(self):
        # Initialize Engine
        lmp = pyr.get_lammps_engine()

        # Optimize Structure
        structure_opt = pyr.optimize_structure(
            lmp=lmp,
            structure=self.structure.copy(),
            potential_dataframe=self.df_pot_selected
        )
        self.assertEqual(len(structure_opt), sum(self.count_lst))

        # Calculate Elastic Constants
        elastic_matrix = pyr.calculate_elastic_constants(
            lmp=lmp,
            structure=structure_opt,
            potential_dataframe=self.df_pot_selected,
            num_of_point=5,
            eps_range=0.005,
            sqrt_eta=True,
            fit_order=2
        )
        self.assertTrue(all(validate_elastic_constants(elastic_matrix=elastic_matrix)))

        # Finalize
        lmp.close()

    def test_example_elastic_constants_with_separate_executable(self):
        # Optimize Structure
        structure_opt = pyr.optimize_structure(
            structure=self.structure.copy(),
            potential_dataframe=self.df_pot_selected
        )
        self.assertEqual(len(structure_opt), sum(self.count_lst))

        # Calculate Elastic Constants
        elastic_matrix = pyr.calculate_elastic_constants(
            structure=structure_opt,
            potential_dataframe=self.df_pot_selected,
            num_of_point=5,
            eps_range=0.005,
            sqrt_eta=True,
            fit_order=2
        )
        self.assertTrue(all(validate_elastic_constants(elastic_matrix=elastic_matrix)))

    def test_example_elastic_constants_with_statement(self):
        with pyr.get_lammps_engine() as lmp:
            # Optimize Structure
            structure_opt = pyr.optimize_structure(
                lmp=lmp,
                structure=self.structure.copy(),
                potential_dataframe=self.df_pot_selected
            )

            # Calculate Elastic Constants
            elastic_matrix = pyr.calculate_elastic_constants(
                lmp=lmp,
                structure=structure_opt,
                potential_dataframe=self.df_pot_selected,
                num_of_point=5,
                eps_range=0.005,
                sqrt_eta=True,
                fit_order=2
            )

        self.assertEqual(len(structure_opt), sum(self.count_lst))
        self.assertTrue(all(validate_elastic_constants(elastic_matrix=elastic_matrix)))

    def test_example_elastic_constants_with_minimization(self):
        with pyr.get_lammps_engine() as lmp:
            elastic_matrix = pyr.calculate_elastic_constants_with_minimization(
                lmp=lmp,
                structure=self.structure.copy(),
                potential_dataframe=self.df_pot_selected,
                num_of_point=5,
                eps_range=0.005,
                sqrt_eta=True,
                fit_order=2
            )

        self.assertTrue(all(validate_elastic_constants(elastic_matrix=elastic_matrix)))


if __name__ == '__main__':
    unittest.main()



from pympipool import Pool
from pyiron_lammps.wrapper import PyironLammpsLibrary
from pyiron_lammps.calculation import (
    optimize_structure,
    calculate_elastic_constants,
    calculate_elastic_constants_with_minimization,
)


def _get_lammps_mpi(enable_mpi=True):
    if enable_mpi:
        # To get the right instance of MPI.COMM_SELF it is necessary to import it inside the function.
        from mpi4py import MPI

        return PyironLammpsLibrary(
            working_directory=None,
            cores=1,
            comm=MPI.COMM_SELF,
            logger=None,
            log_file=None,
            library=None,
            diable_log_file=True,
        )
    else:
        return PyironLammpsLibrary(
            working_directory=None,
            cores=1,
            comm=None,
            logger=None,
            log_file=None,
            library=None,
            diable_log_file=True,
        )


def _parallel_execution(function, input_parameter_lst, cores=1):
    if cores == 1:
        return [
            function(input_parameter=input_parameter + [False])
            for input_parameter in input_parameter_lst
        ]
    elif cores > 1:
        with Pool(cores=cores) as p:
            return p.map(
                function=function,
                lst=[
                    input_parameter + [True] for input_parameter in input_parameter_lst
                ],
            )
    else:
        raise ValueError("The number of cores has to be a positive integer.")


def _optimize_structure_serial(input_parameter):
    structure, potential_dataframe, enable_mpi = input_parameter
    return optimize_structure(
        lmp=_get_lammps_mpi(enable_mpi=enable_mpi),
        structure=structure,
        potential_dataframe=potential_dataframe,
    )


def _calculate_elastic_constants_serial(input_parameter):
    (
        structure,
        potential_dataframe,
        num_of_point,
        eps_range,
        sqrt_eta,
        fit_order,
        enable_mpi,
    ) = input_parameter
    return calculate_elastic_constants(
        lmp=_get_lammps_mpi(enable_mpi=enable_mpi),
        structure=structure,
        potential_dataframe=potential_dataframe,
        num_of_point=num_of_point,
        eps_range=eps_range,
        sqrt_eta=sqrt_eta,
        fit_order=fit_order,
    )


def _calculate_elastic_constants_with_minimization_serial(input_parameter):
    (
        structure,
        potential_dataframe,
        num_of_point,
        eps_range,
        sqrt_eta,
        fit_order,
        enable_mpi,
    ) = input_parameter
    return calculate_elastic_constants_with_minimization(
        lmp=_get_lammps_mpi(enable_mpi=enable_mpi),
        structure=structure,
        potential_dataframe=potential_dataframe,
        num_of_point=num_of_point,
        eps_range=eps_range,
        sqrt_eta=sqrt_eta,
        fit_order=fit_order,
    )


def optimize_structure_parallel(structure_list, potential_dataframe, cores=1):
    return _parallel_execution(
        function=_optimize_structure_serial,
        input_parameter_lst=[
            [structure, potential_dataframe] for structure in structure_list
        ],
        cores=cores,
    )


def calculate_elastic_constants_parallel(
    structure_list,
    potential_dataframe,
    num_of_point=5,
    eps_range=0.005,
    sqrt_eta=True,
    fit_order=2,
    cores=1,
):
    return _parallel_execution(
        function=_calculate_elastic_constants_serial,
        input_parameter_lst=[
            [
                structure,
                potential_dataframe,
                num_of_point,
                eps_range,
                sqrt_eta,
                fit_order,
            ]
            for structure in structure_list
        ],
        cores=cores,
    )


def calculate_elastic_constants_with_minimization_parallel(
    structure_list,
    potential_dataframe,
    num_of_point=5,
    eps_range=0.005,
    sqrt_eta=True,
    fit_order=2,
    cores=1,
):
    return _parallel_execution(
        function=_calculate_elastic_constants_with_minimization_serial,
        input_parameter_lst=[
            [
                structure,
                potential_dataframe,
                num_of_point,
                eps_range,
                sqrt_eta,
                fit_order,
            ]
            for structure in structure_list
        ],
        cores=cores,
    )



from pyiron_lammps.helpers import (
    generate_sqs_structure,
    get_ase_bulk,
    get_lammps_engine,
    get_potential_dataframe,
)
from pyiron_lammps.calculation import (
    optimize_structure,
    calculate_elastic_constants,
    calculate_elastic_constants_with_minimization,
)
from pyiron_lammps.parallel import (
    optimize_structure_parallel,
    calculate_elastic_constants_parallel,
    calculate_elastic_constants_with_minimization_parallel,
)
